#!/usr/bin/perl

#
# ============================== SUMMARY =====================================
#
# Program   : meensatek.pl
# Version   : 1.3
# Date      : 04 May 2015
# Copyright : MENSATEK
# Summary   : Envío de alertas SMS con nagios
# License   : Uso con cuenta de MENSATEK
#
# ============================= INSTALACIÓN ====================================
# 
# Copiar este fichero en el directorio de plugins de nagios
# En Centos sería en /usr/lib/nagios/libexec (32 bit) 
# o /usr/lib64/nagios/libexec (64 bit) 
# En debian y ubuntu : /usr/local/nagios/libexec/ etc...
# después de copiarlo, hazlo ejecutable con 
# sudo chmod 755 mensatek.pl 
# Asegúrate de que tienes SSL habilitado. En caso contrario ejecuta:
# perl -MCPAN -e 'install Crypt::SSLeay'
# Chequear funcionamiento: Desde el directorio de plugins (donde has copiado mensatek.pl) ejecuta
# ./mensatek.pl -u tucorreoRegistradoEnMensatek.com -p tuContraseñaEnMensatek -t TuTeléfono -f ALERTA -m "El mensaje"
#  En caso de no funcionar, contacta con soporte indicando el error que te aparezca. Si el resultado es normal pero aparece un Res=-n
# siendo n un número, verifica en el PDF de la API a qué corresponde ese error (por ejemplo Res:-1 es un error de usuario/contraseña).
# Una vez que funcione pasamos a instalar el plugin:
# INSTALACION DEL PLUGIN EN NAGIOS 
#
# 1. Crear los comandos de notificación de alertas en nagios.  (en commands.cfg ubicado habitualmente en /usr/local/nagios/etc/objects/commands.cfg)
# Acuérdate de cambiar 
# CORREO_REGISTRADO_EN_MENSATEK por tu correo registrado en Mensatek
# PASSWORD_ES_MENSATEK por tu contraseña en Mensatek
# REMITENTE por el remitente que quieras (MENSATEK.ES te hará un descuento sobre el mensaje pero puedes poner lo que quieras, NAGIOS, ALERTA, etc.. hasta 11 caracteres)
# PONER_TELEFONOS por los teléfonos a informar de las alertas, en formato internacional y separados por punto y coma si hat varios. Ejemplo: 34600000001;34601000002;34600000003
#
# define command{
#       command_name    service-notify-by-sms
#       command_line    $USER1$/mensatek.pl -u CORREO_REGISTRADO_EN_MENSATEK -p PASSWORD_ES_MENSATEK -t PONER_TELEFONOS -f REMITENTE -m "Service: $SERVICEDESC$\\nHost: $HOSTNAME$\\nAddress: $HOSTADDRESS$\\nState: $SERVICESTATE$\\nInfo: $SERVICEOUTPUT$\\nDate: $LONGDATETIME$"
# }
#
# define command{
#       command_name    host-notify-by-sms
#       command_line    $USER1$/mensatek.pl -u CORREO_REGISTRADO_EN_MENSATEK -p PASSWORD_ES_MENSATEK -t PONER_TELEFONOS -f REMITENTE -m "Host $HOSTNAME$ is $HOSTSTATE$\\nInfo: $HOSTOUTPUT$\\nTime: $LONGDATETIME$"
# }
#
#
# 2. OPCION 1: En los contactos de nagios (Normalmente en contacts.cfg) añadir
#    Los comandos de notificación:
#
#    service_notification_commands      service-notify-by-sms
#    host_notification_commands         host-notify-by-sms
#
# OPCION 2: COMO ALTERNATIVA, Si somos nagiosadmin (lo más normal) estaremos utilizando generic-contact, por lo que editamos ese contacto en :
# /usr/local/nagios/etc/objects/templates.cfg
#
#Veremos algo como: 
#define contact{
#        name                            generic-contact
#        service_notification_period     24x7
#        host_notification_period        24x7
#        service_notification_options    w,u,c,r,f,s
#        host_notification_options       d,u,r,f,s
#        service_notification_commands   notify-service-by-email
#        host_notification_commands      notify-host-by-email
#        register                        0
#        }
#
# Lo editamos y añadimos el comando a ejecutar en caso de alerta:
#
# define contact{
#        name                            generic-contact
#        service_notification_period     24x7
#        host_notification_period        24x7
#        service_notification_options    w,u,c,r,f,s
#        host_notification_options       d,u,r,f,s
#        service_notification_commands   notify-service-by-email,service-notify-by-sms
#        host_notification_commands      notify-host-by-email,host-notify-by-sms
#        register                        0
#        }
#
#
#
#


use strict;
use Getopt::Long;
use LWP;
use URI::Escape;
use HTTP::Request::Common qw(POST);

my $version = '1.1';
my $verbose = undef;
my $username = undef;
my $password = undef;
my $to = undef;
my $from = "ALERTA";
my $message = undef;

sub print_version { print "$0: version $version\n"; exit(1); };
sub verb { my $t=shift; print "VERBOSE: ",$t,"\n" if defined($verbose) ; }
sub print_usage {
        print "Usage: $0 [-v] -u <correo> -t <destino> [-f <remitente>] -m <mensaje>\n";
}

sub help {
        print "\nNotificaciones por SMS. MENSATEK Plugin ", $version, "\n";
        print " MENSATEK - https://www.mensatek.com/\n\n";
        print_usage();
        print <<EOD;
-h, --help
        print this help message
-V, --version
        print version
-v, --verbose
        print extra debugging information
-u, --usernamme=USERNAME
        Correo registrado en MENSATEK
-p, --password=PASSWORD
        Password en MENSATEK
-t, --to=TO
        Movil destino en formato internacional (ej. 34600000001)
-f, --from=FROM (Optional)
        Remitente del mensaje (max 11 chars)
-m, --message=MESSAGE
        contenido del mensaje
EOD
        exit(1);
}

sub check_options {
        Getopt::Long::Configure ("bundling");
        GetOptions(
                'v'     => \$verbose,           'verbose'       => \$verbose,
                'V'     => \&print_version,     'version'       => \&print_version,
                'h'     => \&help,              'help'          => \&help,     
                'u=s'    => \$username,  'username=s'    => \$username,    
                'p=s'    => \$password,'password=s'    => \$password,
                't=s'   => \$to,      'to=s'          => \$to,
                'f=s'   => \$from,  'from=s'        => \$from,
                'm=s'   => \$message,  'message=s'     => \$message
        );

        if (!defined($username))
                { print "ERROR: No username defined!\n"; print_usage(); exit(1); }
        if (!defined($password))
                { print "ERROR: No password defined!\n"; print_usage(); exit(1); }
        if (!defined($to))
                { print "ERROR: No to defined!\n"; print_usage(); exit(1); }
        if (!defined($message))
                { print "ERROR: No message defined!\n"; print_usage(); exit(1); }

        if($to!~/^\d{7,15}$/) {
                { print "ERROR: Invalid to number!\n"; print_usage(); exit(1); }
        }
        verb "Correo = $username";
        verb "Passwd = $password";
        verb "Destinatarios = $to";
        verb "Remitente = $from";
        verb "Mensaje = $message";
}

sub SendSMS {
        my $username = shift;
        my $password = shift;
        my $to = shift;
        my $from = shift;
        my $message = shift;

        # Convert "\n" to real newlines (Nagios seems to eat newlines).
        $message=~s/\\n/\n/g;

        # URL Encode parameters before making the HTTP POST
        $username   = uri_escape($username);
        $password   = uri_escape($password);
        $to         = uri_escape($to);
        $from       = uri_escape($from);
        $message    = uri_escape($message);

        my $result;
        
		my $baseurl = "https://api.mensatek.com/sms/v5/enviar.php";
		my $getvars = "Correo=$username&Passwd=$password&Mensaje=$message&Destinatarios=$to&Remitente=$from";

        verb("Get Data: ".$getvars);

		my $ua = LWP::UserAgent->new();
		$ua->timeout(5);
		$ua->agent('Nagios-SMS-Plugin/'.$version);
                my $req = POST "$baseurl?$getvars";
		my $res = $ua->request($req);
		
		verb("GET Status: ".$res->status_line);
        verb("GET Response: ".$res->content);

	#	if (index($str, $substr) != -1) {
#			print "$str contains $substr\n";
#		} 

        if($res->is_success) {
                if(not index($res->content, "<code>0<")) {
                        print $res->content;
                        $result = 1;
                } else {
                        $result = 0;
                }
        } else {
                $result = 1;
                print $res->status_line;
        }

        return $result;
}


check_options();
my $send_result = SendSMS($username, $password, $to, $from, $message);

exit($send_result);